@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
+ * Copyright (C) 2011 Jiri Techet <techet@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,18 +20,16 @@
 #error "Only <champlain/champlain.h> can be included directly."
 #endif
 
-#ifndef CHAMPLAIN_LAYER_H
-#define CHAMPLAIN_LAYER_H
+#ifndef __CHAMPLAIN_LAYER_H__
+#define __CHAMPLAIN_LAYER_H__
 
-#include <champlain/champlain-defines.h>
-#include <champlain/champlain-base-marker.h>
-
-#include <glib-object.h>
 #include <clutter/clutter.h>
+#include <champlain/champlain-defines.h>
+#include <champlain/champlain-bounding-box.h>
 
 G_BEGIN_DECLS
 
-#define CHAMPLAIN_TYPE_LAYER champlain_layer_get_type()
+#define CHAMPLAIN_TYPE_LAYER champlain_layer_get_type ()
 
 #define CHAMPLAIN_LAYER(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj), CHAMPLAIN_TYPE_LAYER, ChamplainLayer))
@@ -48,32 +46,31 @@ G_BEGIN_DECLS
 #define CHAMPLAIN_LAYER_GET_CLASS(obj) \
   (G_TYPE_INSTANCE_GET_CLASS ((obj), CHAMPLAIN_TYPE_LAYER, ChamplainLayerClass))
 
-typedef struct {
-  ClutterGroup parent;
-} ChamplainLayer;
+typedef struct _ChamplainLayer ChamplainLayer;
+typedef struct _ChamplainLayerClass ChamplainLayerClass;
 
-typedef struct {
-  ClutterGroupClass parent_class;
-} ChamplainLayerClass;
+struct _ChamplainLayer
+{
+  ClutterActor parent;
+};
+
+struct _ChamplainLayerClass
+{
+  ClutterActorClass parent_class;
+
+  void (*set_view)(ChamplainLayer *layer,
+      ChamplainView *view);
+  ChamplainBoundingBox * (*get_bounding_box) (ChamplainLayer *layer);
+};
 
 GType champlain_layer_get_type (void);
 
-ChamplainLayer* champlain_layer_new (void);
 
-void champlain_layer_show (ChamplainLayer *layer);
-void champlain_layer_hide (ChamplainLayer *layer);
+void champlain_layer_set_view (ChamplainLayer *layer,
+    ChamplainView *view);
 
-void champlain_layer_add_marker (ChamplainLayer *layer,
-    ChamplainBaseMarker *marker);
-void champlain_layer_remove_marker (ChamplainLayer *layer,
-    ChamplainBaseMarker *marker);
-
-void champlain_layer_animate_in_all_markers (ChamplainLayer *layer);
-void champlain_layer_animate_out_all_markers (ChamplainLayer *layer);
-
-void champlain_layer_show_all_markers (ChamplainLayer *layer);
-void champlain_layer_hide_all_markers (ChamplainLayer *layer);
+ChamplainBoundingBox *champlain_layer_get_bounding_box (ChamplainLayer *layer);
 
 G_END_DECLS
 
-#endif
+#endif /* __CHAMPLAIN_LAYER_H__ */
