@@ -25,15 +25,26 @@
 
 #include <champlain/champlain-defines.h>
 #include <champlain/champlain-tile.h>
+#include <champlain/champlain-renderer.h>
 
 G_BEGIN_DECLS
 
-#define CHAMPLAIN_TYPE_MAP_SOURCE             (champlain_map_source_get_type ())
-#define CHAMPLAIN_MAP_SOURCE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), CHAMPLAIN_TYPE_MAP_SOURCE, ChamplainMapSource))
-#define CHAMPLAIN_MAP_SOURCE_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), CHAMPLAIN_TYPE_MAP_SOURCE, ChamplainMapSourceClass))
-#define CHAMPLAIN_IS_MAP_SOURCE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CHAMPLAIN_TYPE_MAP_SOURCE))
-#define CHAMPLAIN_IS_MAP_SOURCE_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), CHAMPLAIN_TYPE_MAP_SOURCE))
-#define CHAMPLAIN_MAP_SOURCE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), CHAMPLAIN_TYPE_MAP_SOURCE, ChamplainMapSourceClass))
+#define CHAMPLAIN_TYPE_MAP_SOURCE champlain_map_source_get_type ()
+
+#define CHAMPLAIN_MAP_SOURCE(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST ((obj), CHAMPLAIN_TYPE_MAP_SOURCE, ChamplainMapSource))
+
+#define CHAMPLAIN_MAP_SOURCE_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST ((klass), CHAMPLAIN_TYPE_MAP_SOURCE, ChamplainMapSourceClass))
+
+#define CHAMPLAIN_IS_MAP_SOURCE(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CHAMPLAIN_TYPE_MAP_SOURCE))
+
+#define CHAMPLAIN_IS_MAP_SOURCE_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_TYPE ((klass), CHAMPLAIN_TYPE_MAP_SOURCE))
+
+#define CHAMPLAIN_MAP_SOURCE_GET_CLASS(obj) \
+  (G_TYPE_INSTANCE_GET_CLASS ((obj), CHAMPLAIN_TYPE_MAP_SOURCE, ChamplainMapSourceClass))
 
 typedef struct _ChamplainMapSourcePrivate ChamplainMapSourcePrivate;
 
@@ -61,21 +72,17 @@ struct _ChamplainMapSourceClass
 {
   GInitiallyUnownedClass parent_class;
 
-  const gchar * (*get_id) (ChamplainMapSource *map_source);
-  const gchar * (*get_name) (ChamplainMapSource *map_source);
-  const gchar * (*get_license) (ChamplainMapSource *map_source);
-  const gchar * (*get_license_uri) (ChamplainMapSource *map_source);
-  guint (*get_min_zoom_level) (ChamplainMapSource *map_source);
-  guint (*get_max_zoom_level) (ChamplainMapSource *map_source);
-  guint (*get_tile_size) (ChamplainMapSource *map_source);
-  ChamplainMapProjection (*get_projection) (ChamplainMapSource *map_source);
+  const gchar * (*get_id)(ChamplainMapSource *map_source);
+  const gchar * (*get_name)(ChamplainMapSource *map_source);
+  const gchar * (*get_license)(ChamplainMapSource *map_source);
+  const gchar * (*get_license_uri)(ChamplainMapSource *map_source);
+  guint (*get_min_zoom_level)(ChamplainMapSource *map_source);
+  guint (*get_max_zoom_level)(ChamplainMapSource *map_source);
+  guint (*get_tile_size)(ChamplainMapSource *map_source);
+  ChamplainMapProjection (*get_projection)(ChamplainMapSource *map_source);
 
-  void (*fill_tile) (ChamplainMapSource *map_source,
-                     ChamplainTile *tile);
-
-  void (*on_set_next_source) (ChamplainMapSource *map_source,
-                              ChamplainMapSource *next_source,
-                              ChamplainMapSource *new_next_source);
+  void (*fill_tile)(ChamplainMapSource *map_source,
+      ChamplainTile *tile);
 };
 
 GType champlain_map_source_get_type (void);
@@ -84,27 +91,31 @@ ChamplainMapSource *champlain_map_source_get_next_source (ChamplainMapSource *ma
 void champlain_map_source_set_next_source (ChamplainMapSource *map_source,
     ChamplainMapSource *next_source);
 
-const gchar * champlain_map_source_get_id (ChamplainMapSource *map_source);
-const gchar * champlain_map_source_get_name (ChamplainMapSource *map_source);
-const gchar * champlain_map_source_get_license (ChamplainMapSource *map_source);
-const gchar * champlain_map_source_get_license_uri (ChamplainMapSource *map_source);
+ChamplainRenderer *champlain_map_source_get_renderer (ChamplainMapSource *map_source);
+void champlain_map_source_set_renderer (ChamplainMapSource *map_source,
+    ChamplainRenderer *renderer);
+
+const gchar *champlain_map_source_get_id (ChamplainMapSource *map_source);
+const gchar *champlain_map_source_get_name (ChamplainMapSource *map_source);
+const gchar *champlain_map_source_get_license (ChamplainMapSource *map_source);
+const gchar *champlain_map_source_get_license_uri (ChamplainMapSource *map_source);
 guint champlain_map_source_get_min_zoom_level (ChamplainMapSource *map_source);
 guint champlain_map_source_get_max_zoom_level (ChamplainMapSource *map_source);
 guint champlain_map_source_get_tile_size (ChamplainMapSource *map_source);
 ChamplainMapProjection champlain_map_source_get_projection (ChamplainMapSource *map_source);
 
-guint champlain_map_source_get_x (ChamplainMapSource *map_source,
+gdouble champlain_map_source_get_x (ChamplainMapSource *map_source,
     guint zoom_level,
     gdouble longitude);
-guint champlain_map_source_get_y (ChamplainMapSource *map_source,
+gdouble champlain_map_source_get_y (ChamplainMapSource *map_source,
     guint zoom_level,
     gdouble latitude);
 gdouble champlain_map_source_get_longitude (ChamplainMapSource *map_source,
     guint zoom_level,
-    guint x);
+    gdouble x);
 gdouble champlain_map_source_get_latitude (ChamplainMapSource *map_source,
     guint zoom_level,
-    guint y);
+    gdouble y);
 guint champlain_map_source_get_row_count (ChamplainMapSource *map_source,
     guint zoom_level);
 guint champlain_map_source_get_column_count (ChamplainMapSource *map_source,
@@ -120,4 +131,3 @@ void champlain_map_source_fill_tile (ChamplainMapSource *map_source,
 G_END_DECLS
 
 #endif /* _CHAMPLAIN_MAP_SOURCE_H_ */
-
