@@ -28,7 +28,7 @@
 #include "champlain-tile-source.h"
 #include "champlain-enum-types.h"
 
-G_DEFINE_TYPE (ChamplainTileSource, champlain_tile_source, CHAMPLAIN_TYPE_MAP_SOURCE);
+G_DEFINE_ABSTRACT_TYPE (ChamplainTileSource, champlain_tile_source, CHAMPLAIN_TYPE_MAP_SOURCE);
 
 #define GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CHAMPLAIN_TYPE_TILE_SOURCE, ChamplainTileSourcePrivate))
 
@@ -246,7 +246,7 @@ champlain_tile_source_class_init (ChamplainTileSourceClass *klass)
   /**
    * ChamplainTileSource:id:
    *
-   * The name of the tile source
+   * The id of the tile source
    *
    * Since: 0.4
    */
