@@ -27,6 +27,7 @@
 
 #include "champlain-enum-types.h"
 #include "champlain-private.h"
+#include "champlain-marshal.h"
 
 #include <math.h>
 #include <errno.h>
@@ -35,11 +36,30 @@
 #include <gio/gio.h>
 #include <clutter/clutter.h>
 
-G_DEFINE_TYPE (ChamplainTile, champlain_tile, CLUTTER_TYPE_GROUP)
+G_DEFINE_TYPE (ChamplainTile, champlain_tile, CLUTTER_TYPE_ACTOR)
 
 #define GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), CHAMPLAIN_TYPE_TILE, ChamplainTilePrivate))
 
+
+static void paint (ClutterActor *self);
+static void pick (ClutterActor *self, 
+    const ClutterColor *color);
+static void get_preferred_width (ClutterActor *self,
+    gfloat for_height,
+    gfloat *min_width_p,
+    gfloat *natural_width_p);
+static void get_preferred_height (ClutterActor *self,
+    gfloat for_width,
+    gfloat *min_height_p,
+    gfloat *natural_height_p);
+static void allocate (ClutterActor *self,
+    const ClutterActorBox *box,
+    ClutterAllocationFlags flags);
+static void map (ClutterActor *self);
+static void unmap (ClutterActor *self);
+
+
 enum
 {
   PROP_0,
@@ -64,13 +84,15 @@ static guint champlain_tile_signals[LAST_SIGNAL] = { 0, };
 
 struct _ChamplainTilePrivate
 {
-  gint x; /* The x position on the map (in pixels) */
-  gint y; /* The y position on the map (in pixels) */
-  gint size; /* The tile's width and height (only support square tiles */
-  gint zoom_level; /* The tile's zoom level */
+  guint x; /* The x position on the map (in pixels) */
+  guint y; /* The y position on the map (in pixels) */
+  guint size; /* The tile's width and height (only support square tiles */
+  guint zoom_level; /* The tile's zoom level */
 
   ChamplainState state; /* The tile state: loading, validation, done */
-  ClutterActor *content_actor; /* The actual tile actor */
+  /* The tile actor that will be displayed after champlain_tile_display_content () */
+  ClutterActor *content_actor; 
+  ClutterGroup *content_group; /* A group used for the fade in effect */
   gboolean fade_in;
 
   GTimeVal *modified_time; /* The last modified time of the cache */
@@ -88,15 +110,15 @@ champlain_tile_get_property (GObject *object,
   switch (property_id)
     {
     case PROP_X:
-      g_value_set_int (value, champlain_tile_get_x (self));
+      g_value_set_uint (value, champlain_tile_get_x (self));
       break;
 
     case PROP_Y:
-      g_value_set_int (value, champlain_tile_get_y (self));
+      g_value_set_uint (value, champlain_tile_get_y (self));
       break;
 
     case PROP_ZOOM_LEVEL:
-      g_value_set_int (value, champlain_tile_get_zoom_level (self));
+      g_value_set_uint (value, champlain_tile_get_zoom_level (self));
       break;
 
     case PROP_SIZE:
@@ -136,15 +158,15 @@ champlain_tile_set_property (GObject *object,
   switch (property_id)
     {
     case PROP_X:
-      champlain_tile_set_x (self, g_value_get_int (value));
+      champlain_tile_set_x (self, g_value_get_uint (value));
       break;
 
     case PROP_Y:
-      champlain_tile_set_y (self, g_value_get_int (value));
+      champlain_tile_set_y (self, g_value_get_uint (value));
       break;
 
     case PROP_ZOOM_LEVEL:
-      champlain_tile_set_zoom_level (self, g_value_get_int (value));
+      champlain_tile_set_zoom_level (self, g_value_get_uint (value));
       break;
 
     case PROP_SIZE:
@@ -183,6 +205,11 @@ champlain_tile_dispose (GObject *object)
       g_object_unref (priv->content_actor);
       priv->content_actor = NULL;
     }
+  if (priv->content_group)
+    {
+      clutter_actor_unparent (CLUTTER_ACTOR (priv->content_group));
+      priv->content_group = NULL;
+    }
 
   G_OBJECT_CLASS (champlain_tile_parent_class)->dispose (object);
 }
@@ -204,6 +231,7 @@ static void
 champlain_tile_class_init (ChamplainTileClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (ChamplainTilePrivate));
 
@@ -212,6 +240,14 @@ champlain_tile_class_init (ChamplainTileClass *klass)
   object_class->dispose = champlain_tile_dispose;
   object_class->finalize = champlain_tile_finalize;
 
+  actor_class->get_preferred_width = get_preferred_width;
+  actor_class->get_preferred_height = get_preferred_height;
+  actor_class->allocate = allocate;
+  actor_class->paint = paint;
+  actor_class->pick = pick;
+  actor_class->map = map;
+  actor_class->unmap = unmap;
+
   /**
    * ChamplainTile:x:
    *
@@ -221,10 +257,10 @@ champlain_tile_class_init (ChamplainTileClass *klass)
    */
   g_object_class_install_property (object_class,
       PROP_X,
-      g_param_spec_int ("x",
+      g_param_spec_uint ("x",
           "x",
           "The X position of the tile",
-          G_MININT,
+          0,
           G_MAXINT,
           0,
           G_PARAM_READWRITE));
@@ -238,10 +274,10 @@ champlain_tile_class_init (ChamplainTileClass *klass)
    */
   g_object_class_install_property (object_class,
       PROP_Y,
-      g_param_spec_int ("y",
+      g_param_spec_uint ("y",
           "y",
           "The Y position of the tile",
-          G_MININT,
+          0,
           G_MAXINT,
           0,
           G_PARAM_READWRITE));
@@ -255,10 +291,10 @@ champlain_tile_class_init (ChamplainTileClass *klass)
    */
   g_object_class_install_property (object_class,
       PROP_ZOOM_LEVEL,
-      g_param_spec_int ("zoom-level",
+      g_param_spec_uint ("zoom-level",
           "Zoom Level",
           "The zoom level of the tile",
-          G_MININT,
+          0,
           G_MAXINT,
           0,
           G_PARAM_READWRITE));
@@ -344,11 +380,23 @@ champlain_tile_class_init (ChamplainTileClass *klass)
           FALSE,
           G_PARAM_READWRITE));
 
+  /**
+   * ChamplainTile::render-complete:
+   * @self: a #ChamplainTile
+   * @data: the result of the rendering
+   * @size: size of data
+   * @error: TRUE if there was an error during rendering
+   * 
+   * The #ChamplainTile::render-complete signal is emitted when rendering of the tile is
+   * completed by the renderer.
+   *
+   * Since: 0.10
+   */
   champlain_tile_signals[RENDER_COMPLETE] =
     g_signal_new ("render-complete", G_OBJECT_CLASS_TYPE (object_class),
         G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-        g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE,
-        1, G_TYPE_POINTER);
+        _champlain_marshal_VOID__POINTER_UINT_BOOLEAN, G_TYPE_NONE,
+        3, G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_BOOLEAN);
 }
 
 
@@ -369,6 +417,9 @@ champlain_tile_init (ChamplainTile *self)
   priv->fade_in = FALSE;
 
   priv->content_actor = NULL;
+  priv->content_group = CLUTTER_GROUP (clutter_group_new ());
+  clutter_actor_set_parent (CLUTTER_ACTOR (priv->content_group), CLUTTER_ACTOR (self));
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
 }
 
 
@@ -388,6 +439,99 @@ champlain_tile_new (void)
 }
 
 
+static void
+paint (ClutterActor *self)
+{
+  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  
+  clutter_actor_paint (CLUTTER_ACTOR (priv->content_group));
+}
+
+
+static void
+pick (ClutterActor *self, 
+    const ClutterColor *color)
+{
+  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+
+  CLUTTER_ACTOR_CLASS (champlain_tile_parent_class)->pick (self, color);
+
+  clutter_actor_paint (CLUTTER_ACTOR (priv->content_group));
+}
+
+
+static void
+get_preferred_width (ClutterActor *self,
+    gfloat for_height,
+    gfloat *min_width_p,
+    gfloat *natural_width_p)
+{
+  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+
+  clutter_actor_get_preferred_width (CLUTTER_ACTOR (priv->content_group),
+      for_height,
+      min_width_p,
+      natural_width_p);
+}
+
+
+static void
+get_preferred_height (ClutterActor *self,
+    gfloat for_width,
+    gfloat *min_height_p,
+    gfloat *natural_height_p)
+{
+  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+
+  clutter_actor_get_preferred_height (CLUTTER_ACTOR (priv->content_group),
+      for_width,
+      min_height_p,
+      natural_height_p);
+}
+
+
+static void
+allocate (ClutterActor *self,
+    const ClutterActorBox *box,
+    ClutterAllocationFlags flags)
+{
+  ClutterActorBox child_box;
+
+  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+
+  CLUTTER_ACTOR_CLASS (champlain_tile_parent_class)->allocate (self, box, flags);
+
+  child_box.x1 = 0;
+  child_box.x2 = box->x2 - box->x1;
+  child_box.y1 = 0;
+  child_box.y2 = box->y2 - box->y1;
+
+  clutter_actor_allocate (CLUTTER_ACTOR (priv->content_group), &child_box, flags);
+}
+
+
+static void
+map (ClutterActor *self)
+{
+  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+
+  CLUTTER_ACTOR_CLASS (champlain_tile_parent_class)->map (self);
+
+  clutter_actor_map (CLUTTER_ACTOR (priv->content_group));
+}
+
+
+static void
+unmap (ClutterActor *self)
+{
+  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+
+  CLUTTER_ACTOR_CLASS (champlain_tile_parent_class)->unmap (self);
+
+  clutter_actor_unmap (CLUTTER_ACTOR (priv->content_group));
+}
+
+
 /**
  * champlain_tile_get_x:
  * @self: the #ChamplainTile
@@ -398,7 +542,7 @@ champlain_tile_new (void)
  *
  * Since: 0.4
  */
-gint
+guint
 champlain_tile_get_x (ChamplainTile *self)
 {
   g_return_val_if_fail (CHAMPLAIN_TILE (self), 0);
@@ -417,7 +561,7 @@ champlain_tile_get_x (ChamplainTile *self)
  *
  * Since: 0.4
  */
-gint
+guint
 champlain_tile_get_y (ChamplainTile *self)
 {
   g_return_val_if_fail (CHAMPLAIN_TILE (self), 0);
@@ -436,7 +580,7 @@ champlain_tile_get_y (ChamplainTile *self)
  *
  * Since: 0.4
  */
-gint
+guint
 champlain_tile_get_zoom_level (ChamplainTile *self)
 {
   g_return_val_if_fail (CHAMPLAIN_TILE (self), 0);
@@ -494,7 +638,7 @@ champlain_tile_get_state (ChamplainTile *self)
  */
 void
 champlain_tile_set_x (ChamplainTile *self,
-    gint x)
+    guint x)
 {
   g_return_if_fail (CHAMPLAIN_TILE (self));
 
@@ -515,7 +659,7 @@ champlain_tile_set_x (ChamplainTile *self,
  */
 void
 champlain_tile_set_y (ChamplainTile *self,
-    gint y)
+    guint y)
 {
   g_return_if_fail (CHAMPLAIN_TILE (self));
 
@@ -536,7 +680,7 @@ champlain_tile_set_y (ChamplainTile *self,
  */
 void
 champlain_tile_set_zoom_level (ChamplainTile *self,
-    gint zoom_level)
+    guint zoom_level)
 {
   g_return_if_fail (CHAMPLAIN_TILE (self));
 
@@ -606,10 +750,10 @@ champlain_tile_set_state (ChamplainTile *self,
  * Since: 0.4
  */
 ChamplainTile *
-champlain_tile_new_full (gint x,
-    gint y,
+champlain_tile_new_full (guint x,
+    guint y,
     guint size,
-    gint zoom_level)
+    guint zoom_level)
 {
   return g_object_new (CHAMPLAIN_TYPE_TILE, "x", x, "y", y, "zoom-level",
       zoom_level, "size", size, NULL);
@@ -705,7 +849,8 @@ champlain_tile_set_etag (ChamplainTile *self,
  * @self: the #ChamplainTile
  * @actor: the new content
  *
- * Sets the tile's content
+ * Sets the tile's content. To also disppay the tile, you have to call
+ * champlain_tile_display_content() in addition.
  *
  * Since: 0.4
  */
@@ -730,11 +875,21 @@ champlain_tile_set_content (ChamplainTile *self,
 static void
 fade_in_completed (G_GNUC_UNUSED ClutterAnimation *animation, ChamplainTile *self)
 {
-  if (clutter_group_get_n_children (CLUTTER_GROUP (self)) > 1)
-    clutter_actor_destroy (clutter_group_get_nth_child (CLUTTER_GROUP (self), 0));
+  ChamplainTilePrivate *priv = self->priv;
+
+  if (clutter_group_get_n_children (CLUTTER_GROUP (priv->content_group)) > 1)
+    clutter_actor_destroy (clutter_group_get_nth_child (CLUTTER_GROUP (priv->content_group), 0));
 }
 
 
+/**
+ * champlain_tile_display_content:
+ * @self: the #ChamplainTile
+ *
+ * Displays the tile's content.
+ *
+ * Since: 0.8
+ */
 void
 champlain_tile_display_content (ChamplainTile *self)
 {
@@ -746,12 +901,13 @@ champlain_tile_display_content (ChamplainTile *self)
   if (!priv->content_actor)
     return;
 
-  clutter_actor_set_opacity (priv->content_actor, 0);
-  clutter_container_add_actor (CLUTTER_CONTAINER (self), priv->content_actor);
+  clutter_container_add_actor (CLUTTER_CONTAINER (priv->content_group), priv->content_actor);
 
+  clutter_actor_set_opacity (CLUTTER_ACTOR (self), 0);
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
   if (priv->fade_in)
     {
-      animation = clutter_actor_animate (priv->content_actor,
+      animation = clutter_actor_animate (CLUTTER_ACTOR (self),
           CLUTTER_EASE_IN_CUBIC,
           500,
           "opacity", 255,
@@ -759,7 +915,7 @@ champlain_tile_display_content (ChamplainTile *self)
     }
   else
     {
-      animation = clutter_actor_animate (priv->content_actor,
+      animation = clutter_actor_animate (CLUTTER_ACTOR (self),
           CLUTTER_LINEAR,
           150,
           "opacity", 255,
@@ -779,7 +935,7 @@ champlain_tile_display_content (ChamplainTile *self)
  *
  * Gets the tile's content actor.
  *
- * Returns: the tile's content, this actor will change each time the tile's content changes.
+ * Returns: (transfer none): the tile's content, this actor will change each time the tile's content changes.
  * You should not unref this content, it is owned by the tile.
  *
  * Since: 0.4
