@@ -35,12 +35,16 @@ create_marker ()
   cairo_t *cr;
 
   /* Create the marker */
-  marker = champlain_base_marker_new ();
+  marker = champlain_custom_marker_new ();
 
   /* Static filled circle ----------------------------------------------- */
   bg = clutter_cairo_texture_new (MARKER_SIZE, MARKER_SIZE);
   cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (bg));
 
+  cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+  cairo_paint(cr);
+  cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+
   /* Draw the circle */
   cairo_set_source_rgb (cr, 0, 0, 0);
   cairo_arc (cr, MARKER_SIZE / 2.0, MARKER_SIZE / 2.0, MARKER_SIZE / 2.0, 0, 2 * M_PI);
@@ -102,7 +106,7 @@ double lon = -73.75;
 typedef struct
 {
   ChamplainView *view;
-  ChamplainBaseMarker *marker;
+  ChamplainMarker *marker;
 } GpsCallbackData;
 
 static gboolean
@@ -111,7 +115,7 @@ gps_callback (GpsCallbackData *data)
   lat += 0.005;
   lon += 0.005;
   champlain_view_center_on (data->view, lat, lon);
-  champlain_base_marker_set_position (data->marker, lat, lon);
+  champlain_location_set_location (CHAMPLAIN_LOCATION (data->marker), lat, lon);
   return TRUE;
 }
 
@@ -120,7 +124,7 @@ int
 main (int argc, char *argv[])
 {
   ClutterActor *actor, *marker, *stage;
-  ChamplainLayer *layer;
+  ChamplainMarkerLayer *layer;
   GpsCallbackData callback_data;
 
   g_thread_init (NULL);
@@ -135,22 +139,22 @@ main (int argc, char *argv[])
   clutter_container_add_actor (CLUTTER_CONTAINER (stage), actor);
 
   /* Create the marker layer */
-  layer = champlain_layer_new ();
+  layer = champlain_marker_layer_new_full (CHAMPLAIN_SELECTION_SINGLE);
   clutter_actor_show (CLUTTER_ACTOR (layer));
-  champlain_view_add_layer (CHAMPLAIN_VIEW (actor), layer);
+  champlain_view_add_layer (CHAMPLAIN_VIEW (actor), CHAMPLAIN_LAYER (layer));
 
   /* Create a marker */
   marker = create_marker ();
-  clutter_container_add (CLUTTER_CONTAINER (layer), marker, NULL);
+  champlain_marker_layer_add_marker (layer, CHAMPLAIN_MARKER (marker));
 
   /* Finish initialising the map view */
   g_object_set (G_OBJECT (actor), "zoom-level", 12,
-      "scroll-mode", CHAMPLAIN_SCROLL_MODE_KINETIC, NULL);
+      "kinetic-mode", TRUE, NULL);
   champlain_view_center_on (CHAMPLAIN_VIEW (actor), lat, lon);
 
   /* Create callback that updates the map periodically */
   callback_data.view = CHAMPLAIN_VIEW (actor);
-  callback_data.marker = CHAMPLAIN_BASE_MARKER (marker);
+  callback_data.marker = CHAMPLAIN_MARKER (marker);
 
   g_timeout_add (1000, (GSourceFunc) gps_callback, &callback_data);
 
