@@ -119,14 +119,19 @@ static void
 load_network_map_data (ChamplainNetworkBboxTileSource *source, ChamplainView *view)
 {
   gdouble lat, lon;
+  ChamplainBoundingBox *bbox = champlain_bounding_box_new ();
 
   g_signal_connect (source, "notify::state", G_CALLBACK (data_source_state_changed),
       map_data_state_img);
 
   g_object_get (G_OBJECT (view), "latitude", &lat, "longitude", &lon, NULL);
 
-  champlain_network_bbox_tile_source_load_map_data (source,
-      lon - 0.008, lat - 0.008, lon + 0.008, lat + 0.008);
+  bbox->left = lon - 0.008;
+  bbox->right = lon + 0.008;
+  bbox->bottom = lat - 0.008;
+  bbox->top = lat + 0.008;
+  champlain_network_bbox_tile_source_load_map_data (source, bbox);
+  champlain_bounding_box_free (bbox);
 }
 
 
@@ -451,7 +456,7 @@ map_source_changed (GtkWidget *widget, ChamplainView *view)
         {
           champlain_memphis_renderer_load_rules (CHAMPLAIN_MEMPHIS_RENDERER (renderer), rules[rules_index]);
           champlain_file_tile_source_load_map_data (CHAMPLAIN_FILE_TILE_SOURCE (source), maps[map_index]);
-          gtk_widget_hide_all (memphis_box);
+          gtk_widget_hide (memphis_box);
           gtk_widget_set_no_show_all (memphis_box, FALSE);
           gtk_widget_set_no_show_all (memphis_local_box, FALSE);
           gtk_widget_set_no_show_all (memphis_net_box, TRUE);
@@ -461,7 +466,7 @@ map_source_changed (GtkWidget *widget, ChamplainView *view)
         {
           champlain_memphis_renderer_load_rules (CHAMPLAIN_MEMPHIS_RENDERER (renderer), rules[rules_index]);
           load_network_map_data (CHAMPLAIN_NETWORK_BBOX_TILE_SOURCE (source), view);
-          gtk_widget_hide_all (memphis_box);
+          gtk_widget_hide (memphis_box);
           gtk_widget_set_no_show_all (memphis_box, FALSE);
           gtk_widget_set_no_show_all (memphis_local_box, TRUE);
           gtk_widget_set_no_show_all (memphis_net_box, FALSE);
@@ -469,7 +474,7 @@ map_source_changed (GtkWidget *widget, ChamplainView *view)
         }
       else
         {
-          gtk_widget_hide_all (memphis_box);
+          gtk_widget_hide (memphis_box);
           gtk_widget_set_no_show_all (memphis_box, TRUE);
         }
 
@@ -608,18 +613,18 @@ build_source_combo_box (GtkComboBox *box)
         -1);
 
   factory = champlain_map_source_factory_dup_default ();
-  sources = champlain_map_source_factory_dup_list (factory);
+  sources = champlain_map_source_factory_get_registered (factory);
 
   iter = sources;
   while (iter != NULL)
     {
-      ChamplainMapSourceDesc *desc;
-
-      desc = (ChamplainMapSourceDesc *) iter->data;
+      ChamplainMapSourceDesc *desc = CHAMPLAIN_MAP_SOURCE_DESC (iter->data);
+      const gchar *id = champlain_map_source_desc_get_id (desc);
+      const gchar *name = champlain_map_source_desc_get_name (desc);
 
       gtk_tree_store_append (store, &parent, NULL);
-      gtk_tree_store_set (store, &parent, COL_ID, desc->id,
-          COL_NAME, desc->name, -1);
+      gtk_tree_store_set (store, &parent, COL_ID, id,
+          COL_NAME, name, -1);
 
       iter = g_slist_next (iter);
     }
@@ -756,7 +761,7 @@ main (int argc,
   widget = gtk_champlain_embed_new ();
   champlain_view = gtk_champlain_embed_get_view (GTK_CHAMPLAIN_EMBED (widget));
 
-  g_object_set (G_OBJECT (champlain_view), "scroll-mode", CHAMPLAIN_SCROLL_MODE_KINETIC,
+  g_object_set (G_OBJECT (champlain_view), "kinetic-mode", TRUE,
       "zoom-level", 9, NULL);
 
   gtk_widget_set_size_request (widget, 640, 480);
